@@ -20,7 +20,7 @@ test_that("(sparse) matrix sum template works: fixed M0 and variable M1", {
   ms <- make_mat_sum(M0=M0, M1=M1)
   expect_equivalent(as.matrix(M0 + 4*M1), as.matrix(ms(M1, w1=4)))
   expect_equivalent(as.matrix(M0 + 4*M1), as.matrix(ms(2*M1, w1=2)))
-  M0 <- tcrossprod(rep.int(1,n))
+  M0 <- base_tcrossprod(rep.int(1,n))
   ms <- make_mat_sum(M0=M0, M1=M1)
   expect_is(ms(M1, w1=4), "matrix")
   expect_equivalent(as.matrix(M0 + 4*M1), as.matrix(ms(M1, w1=4)))
@@ -107,7 +107,7 @@ test_that("(sparse) matrix sum template works: fixed M0, variable M1 and M2", {
   ms <- make_mat_sum(M0=M0, M1=M1, M2=M2)
   expect_equivalent(as.matrix(M0 + 2*M1 + 3*M2), as.matrix(ms(M1, M2, w1=2, w2=3)))
   expect_equivalent(as.matrix(M0 + 2*M1 + 3*M2), as.matrix(ms(4*M1, 3*M2, w1=0.5)))
-  M0 <- tcrossprod(rep.int(1,n))
+  M0 <- base_tcrossprod(rep.int(1,n))
   ms <- make_mat_sum(M0=M0, M1=M1, M2=M2)
   expect_is(ms(M1, M2), "matrix")
   expect_equivalent(as.matrix(M0 + 2*M1 + 3*M2), as.matrix(ms(M1, M2, w1=2, w2=3)))
