@@ -101,5 +101,16 @@ test_that("Parallel functionality works", {
   pred <- predict(sim, fun.=f, labels="median", ppcheck=TRUE, cl=cl, show.progress=FALSE, verbose=FALSE)
   expect_length(attr(pred, "ppp"), 1L)
   expect_between(attr(pred, "ppp"), 0.8*attr(pred0, "ppp"), 1.2*attr(pred0, "ppp"))
+  WAIC <- compute_WAIC(sim, diagnostic=TRUE, show.progress=FALSE)
+  WAICp <- compute_WAIC(sim, diagnostic=TRUE, cl=cl)
+  suppressWarnings(WAICloo <- waic(sim))
+  expect_equal(WAIC$WAIC2, WAICp$WAIC2, tolerance=1e-6)
+  expect_equal(WAIC$WAIC2, WAICloo$estimates["waic", "Estimate"], tolerance=1e-4)
+  expect_equal(WAIC$p_WAIC2, WAICloo$estimates["p_waic", "Estimate"], tolerance=1e-4)
+  expect_equal(WAIC$lppd, WAICp$lppd, tolerance=1e-6)
+  expect_equal(WAIC$pwaic, WAICp$pwaic, tolerance=1e-6)
+  expect_equal(WAICloo$pointwise[, "elpd_waic"], WAIC$lppd - WAIC$pwaic, tolerance=1e-4)
+  expect_equal(WAIC$WAIC1, WAIC$WAIC2, tolerance=0.2)
+  expect_equal(WAIC$p_WAIC1, WAIC$p_WAIC2, tolerance=0.4)
   stop_cluster(sim[["_cluster"]])
 })
